// My test

// Main entry point for random map script
void main(void)
{
   // Text
   rmSetStatusText("",0.01);

   rmCreateTrigger("Day");
   rmCreateTrigger("Dawn");

   rmSwitchToTrigger(rmTriggerID("Day"));
   rmSetTriggerActive(true);
   rmAddTriggerCondition("Timer");
   rmSetTriggerConditionParamInt("Param1", 60);
   rmAddTriggerEffect("Set Lighting");
   rmSetTriggerEffectParam("SetName", "Default");
   rmSetTriggerEffectParamInt("FadeTime", 30);
   rmAddTriggerEffect("Fire Event");
   rmSetTriggerEffectParamInt("EventID", rmTriggerID("Dawn"));

   rmSwitchToTrigger(rmTriggerID("Dawn"));
   rmSetTriggerActive(false);
   rmAddTriggerCondition("Timer");
   rmSetTriggerConditionParamInt("Param1", 120);
   rmAddTriggerEffect("Set Lighting");
   rmSetTriggerEffectParam("SetName", "Dawn");
   rmSetTriggerEffectParamInt("FadeTime", 30);
   rmAddTriggerEffect("Fire Event");
   rmSetTriggerEffectParamInt("EventID", rmTriggerID("Day"));

   int skin=rmRandInt(0,2);
   float connect=rmRandFloat(0,1);
   float border=rmRandFloat(0,1);
   int herd=rmRandInt(0,2);

   // Set size.
   int playerTiles=10000;
   if(cMapSize == 1)
   {
      playerTiles = 12400;
      rmEchoInfo("Large map");
   }
   int size=2.8*sqrt(cNumberNonGaiaPlayers*playerTiles/0.9);
   rmEchoInfo("Map size="+size+"m x "+size+"m");
   rmSetMapSize(size, size);

   // Set up default water.
   rmSetSeaLevel(0.0);
   if(skin==0)
   rmSetSeaType("mediterranean sea");
   if(skin==1)
   rmSetSeaType("watering hole");
   if(skin==2)
   rmSetSeaType("norwegian sea");

   // Init map.

   if(border<0.8)
   rmTerrainInitialize("water");
   else
   {
   if(skin==0)
   {
   rmSetSeaType("greek river");
   rmTerrainInitialize("water");
   }
   if(skin==1)
   {
   rmSetSeaType("egyptian nile shallow");
   rmTerrainInitialize("water");
   }
   if(skin==2)
   rmTerrainInitialize("IceC");
   }
   rmSetLightingSet("dawn");

   // Define some classes.
   int classPlayer=rmDefineClass("player");
   rmDefineClass("starting settlement");
   int classIce=rmDefineClass("ice");
   int classBonusHuntable=rmDefineClass("bonus huntable");
   rmDefineClass("classHill");
   int connectionClass=rmDefineClass("connection");

   // Text
   rmSetStatusText("",0.05);

    // -------------Define constraints

   // Player area constraint.
   int playerConstraint=rmCreateClassDistanceConstraint("stay away from players", classPlayer, 10.0);
   int connectionEdgeConstraint=rmCreateBoxConstraint("connections avoid edge of map", rmXTilesToFraction(16), rmZTilesToFraction(16), 1.0-rmXTilesToFraction(16), 1.0-rmZTilesToFraction(16));

   // Connection
   int connectionConstraint=rmCreateClassDistanceConstraint("stay away from connection", connectionClass, 4.0);

   // Create a edge of map constraint.
   int edgeConstraint=rmCreateBoxConstraint("edge of map", rmXTilesToFraction(20), rmZTilesToFraction(20), 1.0-rmXTilesToFraction(20), 1.0-rmZTilesToFraction(20), 0.01);

   // Settlement constraint.
   int shortAvoidSettlement=rmCreateTypeDistanceConstraint("objects avoid TC by short distance", "AbstractSettlement", 20.0);
   int farAvoidSettlement=rmCreateTypeDistanceConstraint("objects avoid TC by long distance", "AbstractSettlement", 50.0);
   int farStartingSettleConstraint=rmCreateClassDistanceConstraint("objects avoid player TCs", rmClassID("starting settlement"), 40.0);

   // Tower constraint.
   int avoidTower=rmCreateTypeDistanceConstraint("towers avoid towers", "tower", 24.0);

   // Gold
   int avoidGold=rmCreateTypeDistanceConstraint("avoid gold", "gold", 30.0);
   int shortAvoidGold=rmCreateTypeDistanceConstraint("short avoid gold", "gold", 10.0);

   // Herd animals
   int avoidHerdable=rmCreateTypeDistanceConstraint("avoid herdable", "herdable", 20.0);
   int avoidFood=rmCreateTypeDistanceConstraint("avoid food", "food", 6.0);
   int avoidPredator=rmCreateTypeDistanceConstraint("avoid predator", "animalPredator", 20.0);

   //Hunted animals
   int avoidBonusHuntable=rmCreateClassDistanceConstraint("avoid bonus huntable", classBonusHuntable, 30.0);

   // Avoid impassable land
   int avoidImpassableLand=rmCreateTerrainDistanceConstraint("avoid impassable land", "land", false, 10.0);
   int shortAvoidImpassableLand=rmCreateTerrainDistanceConstraint("short avoid impassable land", "land", false, 5.0);
   int avoidBuildings=rmCreateTypeDistanceConstraint("avoid buildings", "Building", 15.0);
   int shortHillConstraint=rmCreateClassDistanceConstraint("patches vs. hill", rmClassID("classHill"), 10.0);
   int sharkLand = rmCreateTerrainDistanceConstraint("shark land", "land", true, 20.0);

    // Stay near shore
   int nearShore=rmCreateTerrainMaxDistanceConstraint("near shore", "water", true, 12.0);

   // -------------Define objects
   // Close Objects

   int startingSettlementID=rmCreateObjectDef("Starting settlement");
   rmAddObjectDefItem(startingSettlementID, "Settlement Level 1", 1, 0.0);
   rmAddObjectDefToClass(startingSettlementID, rmClassID("starting settlement"));
   rmSetObjectDefMinDistance(startingSettlementID, 0.0);
   rmSetObjectDefMaxDistance(startingSettlementID, 0.0);

   // towers avoid other towers
   int startingTowerID=rmCreateObjectDef("Starting tower");
   rmAddObjectDefItem(startingTowerID, "tower", 1, 0.0);
   rmSetObjectDefMinDistance(startingTowerID, 22.0);
   rmSetObjectDefMaxDistance(startingTowerID, 28.0);
   rmAddObjectDefConstraint(startingTowerID, avoidTower);

   // gold avoids gold
   int startingGoldID=rmCreateObjectDef("Starting gold");
   rmAddObjectDefItem(startingGoldID, "Gold mine small", 1, 0.0);
   rmSetObjectDefMinDistance(startingGoldID, 20.0);
   rmSetObjectDefMaxDistance(startingGoldID, 25.0);
   rmAddObjectDefConstraint(startingGoldID, avoidGold);
   rmAddObjectDefConstraint(startingGoldID, avoidImpassableLand);

   // Birds
   int farhawkID=rmCreateObjectDef("far hawks");
   if(skin==1)
   rmAddObjectDefItem(farhawkID, "vulture", 1, 0.0);
   else
   rmAddObjectDefItem(farhawkID, "hawk", 1, 0.0);
   rmSetObjectDefMinDistance(farhawkID, 0.0);
   rmSetObjectDefMaxDistance(farhawkID, rmXFractionToMeters(0.5));

   // Text
   rmSetStatusText("",0.10);

   // Relics avoid TCs
   int relicID=rmCreateObjectDef("relic");
   rmAddObjectDefItem(relicID, "relic", 1, 0.0);
   rmSetObjectDefMinDistance(relicID, 60.0);
   rmSetObjectDefMaxDistance(relicID, 150.0);
   rmAddObjectDefConstraint(relicID, rmCreateTypeDistanceConstraint("relic vs relic", "relic", 70.0));
   rmAddObjectDefConstraint(relicID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(relicID, shortAvoidImpassableLand);

   int closeCowsID=rmCreateObjectDef("close Cows");
   if(herd==0)
   rmAddObjectDefItem(closeCowsID, "cow", rmRandInt(2,4), 6.0);
   if(herd==1)
   rmAddObjectDefItem(closeCowsID, "pig", rmRandInt(3,4), 6.0);
   if(herd==2)
   rmAddObjectDefItem(closeCowsID, "goat", rmRandInt(3,4), 6.0);
   rmSetObjectDefMinDistance(closeCowsID, 25.0);
   rmSetObjectDefMaxDistance(closeCowsID, 30.0);
   rmAddObjectDefConstraint(closeCowsID, avoidImpassableLand);
   rmAddObjectDefConstraint(closeCowsID, avoidFood);

   int closeBerriesID=rmCreateObjectDef("close berries");
   if(rmRandFloat(0,1)<0.8)
      rmAddObjectDefItem(closeBerriesID, "berry bush", rmRandInt(7,9), 4.0);
   else
      rmAddObjectDefItem(closeBerriesID, "berry bush", rmRandInt(4,6), 2.0);
   rmSetObjectDefMinDistance(closeBerriesID, 20.0);
   rmSetObjectDefMaxDistance(closeBerriesID, 25.0);
   rmAddObjectDefConstraint(closeBerriesID, avoidImpassableLand);
   rmAddObjectDefConstraint(closeBerriesID, avoidFood);
   
   int closeBoarID=rmCreateObjectDef("close Boar");
   float boarNumber=rmRandFloat(0, 1);
   rmAddObjectDefItem(closeBoarID, "boar", 2, 4.0);
   rmSetObjectDefMinDistance(closeBoarID, 30.0);
   rmSetObjectDefMaxDistance(closeBoarID, 50.0);
   rmAddObjectDefConstraint(closeBoarID, avoidImpassableLand);
   
   int stragglerTreeID=rmCreateObjectDef("straggler tree");
   if(skin==0)
   rmAddObjectDefItem(stragglerTreeID, "oak tree", 1, 0.0);
   if(skin==1)
   rmAddObjectDefItem(stragglerTreeID, "palm", 1, 0.0);
   if(skin==2)
   rmAddObjectDefItem(stragglerTreeID, "pine snow", 1, 0.0);
   rmSetObjectDefMinDistance(stragglerTreeID, 12.0);
   rmSetObjectDefMaxDistance(stragglerTreeID, 15.0);

    // player fish
   int fishVsFishID=rmCreateTypeDistanceConstraint("fish v fish", "fish", 18.0);
   int fishLand = rmCreateTerrainDistanceConstraint("fish land", "land", true, 6.0);

   int playerFishID=rmCreateObjectDef("owned fish");
   int fishType=rmRandInt(0,2);
   if(fishType==0)
   rmAddObjectDefItem(playerFishID, "fish - salmon", 3, 10.0);
   if(fishType==1)
   rmAddObjectDefItem(playerFishID, "fish - perch", 3, 10.0);
   if(fishType==2)
   rmAddObjectDefItem(playerFishID, "fish - mahi", 3, 10.0);
   rmSetObjectDefMinDistance(playerFishID, 0.0);
   rmSetObjectDefMaxDistance(playerFishID, 100.0);
   rmAddObjectDefConstraint(playerFishID, fishVsFishID);
   rmAddObjectDefConstraint(playerFishID, fishLand);

   // gold avoids gold and Settlements
   int mediumGoldID=rmCreateObjectDef("medium gold");
   rmAddObjectDefItem(mediumGoldID, "Gold mine", 1, 0.0);
   rmSetObjectDefMinDistance(mediumGoldID, 40.0);
   rmSetObjectDefMaxDistance(mediumGoldID, 60.0);
   rmAddObjectDefConstraint(mediumGoldID, avoidGold);
   rmAddObjectDefConstraint(mediumGoldID, shortAvoidSettlement);
   rmAddObjectDefConstraint(mediumGoldID, avoidImpassableLand);
   rmAddObjectDefConstraint(mediumGoldID, farStartingSettleConstraint);

   int numHuntable=rmRandInt(4, 8);
   int huntType=rmRandInt(0,1);

   // Text
   rmSetStatusText("",0.20);

   int mediumDeerID=rmCreateObjectDef("medium deer");
   if(skin==0)
   rmAddObjectDefItem(mediumDeerID, "deer", numHuntable, 3.0);
   if(skin==1)
   {
      if(huntType==0)
      rmAddObjectDefItem(mediumDeerID, "giraffe", numHuntable, 3.0);
      if(huntType==1)
      rmAddObjectDefItem(mediumDeerID, "zebra", numHuntable, 3.0);
   }
   if(skin==2)
   {
      if(huntType==0)
      rmAddObjectDefItem(mediumDeerID, "elk", numHuntable, 3.0);
      if(huntType==1)
      rmAddObjectDefItem(mediumDeerID, "caribou", numHuntable, 3.0);
   }
   rmSetObjectDefMinDistance(mediumDeerID, 60.0);
   rmSetObjectDefMaxDistance(mediumDeerID, 80.0);
   rmAddObjectDefConstraint(mediumDeerID, avoidImpassableLand);
   rmAddObjectDefConstraint(mediumDeerID, avoidFood);
   rmAddObjectDefConstraint(mediumDeerID, farStartingSettleConstraint);

   // hunted avoids hunted and TCs
   int mediumWalrusID=rmCreateObjectDef("medium walrus");
   if(skin==0)
   rmAddObjectDefItem(mediumWalrusID, "water buffalo", rmRandInt(3,6), 8.0);
   if(skin==1)
   rmAddObjectDefItem(mediumWalrusID, "hippo", rmRandInt(3,6), 8.0);
   if(skin==2)
   rmAddObjectDefItem(mediumWalrusID, "walrus", rmRandInt(3,6), 8.0);
   rmSetObjectDefMinDistance(mediumWalrusID, 50);
   rmSetObjectDefMaxDistance(mediumWalrusID, 120);
   rmAddObjectDefToClass(mediumWalrusID, classBonusHuntable);
   rmAddObjectDefConstraint(mediumWalrusID, nearShore);
   rmAddObjectDefConstraint(mediumWalrusID, avoidImpassableLand);
   rmAddObjectDefConstraint(mediumWalrusID, farStartingSettleConstraint);

   // Far Objects

   // gold avoids gold, Settlements and TCs
   int farGoldID=rmCreateObjectDef("far gold");
   rmAddObjectDefItem(farGoldID, "Gold mine", 1, 0.0);
   rmSetObjectDefMinDistance(farGoldID, 80.0);
   rmSetObjectDefMaxDistance(farGoldID, 150.0);
   rmAddObjectDefConstraint(farGoldID, avoidGold);
   rmAddObjectDefConstraint(farGoldID, avoidImpassableLand);
   rmAddObjectDefConstraint(farGoldID, shortAvoidSettlement);
   rmAddObjectDefConstraint(farGoldID, farStartingSettleConstraint);
  
   int farCowID=rmCreateObjectDef("far Cow");
   herd=rmRandInt(0,2);
   if(herd==0)
   rmAddObjectDefItem(farCowID, "cow", rmRandInt(1,2), 5.0);
   if(herd==1)
   rmAddObjectDefItem(farCowID, "pig", rmRandInt(2,4), 5.0);
   if(herd==2)
   rmAddObjectDefItem(farCowID, "goat", rmRandInt(2,3), 5.0);
   rmSetObjectDefMinDistance(farCowID, 80.0);
   rmSetObjectDefMaxDistance(farCowID, 150.0);
   rmAddObjectDefConstraint(farCowID, farStartingSettleConstraint);
   rmAddObjectDefConstraint(farCowID, avoidImpassableLand);
   rmAddObjectDefConstraint(farCowID, avoidHerdable);

   // pick wolves or bears as predators
   // avoid TCs and other animals
   int farPredatorID=rmCreateObjectDef("far predator");
   float predatorSpecies=rmRandFloat(0, 1);
   if(skin==0)
      rmAddObjectDefItem(farPredatorID, "bear", 2, 2.0);
   if(skin==1)
      rmAddObjectDefItem(farPredatorID, "crocodile", 2, 2.0);
   if(skin==2)
      rmAddObjectDefItem(farPredatorID, "polar bear", 2, 2.0);
   rmSetObjectDefMinDistance(farPredatorID, 50.0);
   rmSetObjectDefMaxDistance(farPredatorID, 100.0);
   rmAddObjectDefConstraint(farPredatorID, avoidPredator);
   rmAddObjectDefConstraint(farPredatorID, farStartingSettleConstraint); 
   rmAddObjectDefConstraint(farPredatorID, shortAvoidImpassableLand);
   rmAddObjectDefConstraint(farPredatorID, avoidFood);

   // Text
   rmSetStatusText("",0.30);

   // This map will either use boar or deer as the extra huntable food.

   // hunted avoids hunted and TCs
   int bonusHuntableID=rmCreateObjectDef("bonus huntable");
   int bonusChance=rmRandFloat(0, 1);
   if(skin==0)
   rmAddObjectDefItem(bonusHuntableID, "deer", rmRandInt(4,9), 3.0);
   if(skin==1)
   {
      if(huntType==0)
      rmAddObjectDefItem(bonusHuntableID, "giraffe", rmRandInt(4,9), 3.0);
      if(huntType==1)
      rmAddObjectDefItem(bonusHuntableID, "zebra", rmRandInt(4,9), 3.0);
   }
   if(skin==2)
   {
      if(huntType==0)
      rmAddObjectDefItem(bonusHuntableID, "elk", rmRandInt(4,9), 3.0);
      if(huntType==1)
      rmAddObjectDefItem(bonusHuntableID, "caribou", rmRandInt(4,9), 3.0);
   }
   rmSetObjectDefMinDistance(bonusHuntableID, 0.0);
   rmSetObjectDefMaxDistance(bonusHuntableID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(bonusHuntableID, avoidBonusHuntable);
   rmAddObjectDefToClass(bonusHuntableID, classBonusHuntable);
   rmAddObjectDefConstraint(bonusHuntableID, farStartingSettleConstraint); 
   rmAddObjectDefConstraint(bonusHuntableID, avoidImpassableLand);
   
   // hunted avoids hunted and TCs
   int bonusWalrus=rmCreateObjectDef("bonus walrus");
   if(skin==0)
   rmAddObjectDefItem(bonusWalrus, "water buffalo", rmRandInt(2,5), 8.0);
   if(skin==1)
   rmAddObjectDefItem(bonusWalrus, "hippo", rmRandInt(2,5), 8.0);
   if(skin==2)
   rmAddObjectDefItem(bonusWalrus, "walrus", rmRandInt(2,5), 8.0);
   rmSetObjectDefMinDistance(bonusWalrus, 60.0);
   rmSetObjectDefMaxDistance(bonusWalrus, 150.0);
   rmAddObjectDefConstraint(bonusWalrus, avoidBonusHuntable);
   rmAddObjectDefToClass(bonusWalrus, classBonusHuntable);
   rmAddObjectDefConstraint(bonusWalrus, farStartingSettleConstraint);
   rmAddObjectDefConstraint(bonusWalrus, nearShore);
   
   int randomTreeID=rmCreateObjectDef("random tree");
   if(skin==0)
   rmAddObjectDefItem(randomTreeID, "oak tree", 1, 0.0);
   if(skin==1)
   rmAddObjectDefItem(randomTreeID, "palm", 1, 0.0);
   if(skin==2)
   rmAddObjectDefItem(randomTreeID, "pine snow", 1, 0.0);
   rmSetObjectDefMinDistance(randomTreeID, 0.0);
   rmSetObjectDefMaxDistance(randomTreeID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(randomTreeID, rmCreateTypeDistanceConstraint("random tree", "all", 4.0));
   rmAddObjectDefConstraint(randomTreeID, shortAvoidSettlement);
   rmAddObjectDefConstraint(randomTreeID, avoidImpassableLand);

   // Text
   rmSetStatusText("",0.40);

   // Grow a big continent.
   int centerID=rmCreateArea("center");
   rmSetAreaSize(centerID, 0.5, 0.55);
   rmSetAreaLocation(centerID, 0.5, 0.5);
//   rmSetAreaMinBlobs(centerID, 1);
//   rmSetAreaMaxBlobs(centerID, 5);
//   rmSetAreaMinBlobDistance(centerID, 40.0);
//   rmSetAreaMaxBlobDistance(centerID, 60.0);
   rmSetAreaCoherence(centerID, 0.0);
   rmSetAreaBaseHeight(centerID, 4.0);
   if(skin==0)
   rmSetAreaTerrainType(centerID, "GrassA");
   if(skin==1)
   rmSetAreaTerrainType(centerID, "SandA");
   if(skin==2)
   rmSetAreaTerrainType(centerID, "SnowA");
   rmSetAreaSmoothDistance(centerID, 30);
   rmSetAreaHeightBlend(centerID, 2);
   rmAddAreaConstraint(centerID, rmCreateBoxConstraint("center-edge", 0.05, 0.05, 0.95, 0.95, 0.01));
   rmBuildArea(centerID);

   // Grow a big continent.
   int center2ID=rmCreateArea("center2");
   rmSetAreaSize(center2ID, 0.45, 0.5);
   rmSetAreaLocation(center2ID, 0.5, 0.5);
//   rmSetAreaMinBlobs(center2ID, 1);
//   rmSetAreaMaxBlobs(center2ID, 5);
//   rmSetAreaMinBlobDistance(center2ID, 40.0);
//   rmSetAreaMaxBlobDistance(center2ID, 60.0);
   rmSetAreaCoherence(center2ID, 1.0);
   rmSetAreaBaseHeight(center2ID, 12.0);
   if(skin==0)
   rmSetAreaTerrainType(center2ID, "cliffGreekA");
   if(skin==1)
   rmSetAreaTerrainType(center2ID, "cliffEgyptianA");
   if(skin==2)
   rmSetAreaTerrainType(center2ID, "cliffNorseA");
   rmSetAreaSmoothDistance(center2ID, 30);
   rmSetAreaHeightBlend(center2ID, 2);
   rmAddAreaConstraint(center2ID, rmCreateBoxConstraint("center-edge", 0.05, 0.05, 0.95, 0.95, 0.01));
   rmBuildArea(center2ID);

   // Set up a connection... we'll add all the team areas to it.
   if(connect<0.7)
   {
   int connectionID=rmCreateConnection("passes");
   rmAddConnectionTerrainReplacement(connectionID, "cliffNorseA", "SnowB");
   rmAddConnectionTerrainReplacement(connectionID, "cliffEgyptianA", "SandB");
   rmAddConnectionTerrainReplacement(connectionID, "cliffGreekA", "GrassA"); 
   rmSetConnectionType(connectionID, cConnectAreas, false, 1.0);
   rmSetConnectionWarnFailure(connectionID, false); 
   rmSetConnectionWidth(connectionID, 17, 4);
   rmSetConnectionTerrainCost(connectionID, "cliffNorseA", 5.0);
   rmSetConnectionTerrainCost(connectionID, "cliffNorseB", 3.0);
   rmSetConnectionPositionVariance(connectionID, 0.3);
   rmSetConnectionBaseHeight(connectionID, 0.0);
   rmSetConnectionHeightBlend(connectionID, 2);
   rmAddConnectionToClass(connectionID, connectionClass);
   }

   // Cheesy "circular" placement of players.
   if(cNumberNonGaiaPlayers > 8)
      rmPlacePlayersCircular(0.43, 0.40, rmDegreesToRadians(2.0));
   else
      rmPlacePlayersCircular(0.38, 0.35, rmDegreesToRadians(2.0));

   // Set up player areas.
   float playerFraction=rmAreaTilesToFraction(4000);
   for(i=1; <cNumberPlayers)
   {
      // Create the area.
      int id=rmCreateArea("Player"+i, centerID);
      rmSetPlayerArea(i, id);
      rmSetAreaSize(id, 1.5*playerFraction, 1.7*playerFraction);
      rmAddAreaToClass(id, classPlayer);
      rmSetAreaWarnFailure(id, false);
      rmSetAreaMinBlobs(id, 1);
      rmSetAreaMaxBlobs(id, 5);
      rmSetAreaMinBlobDistance(id, 16.0);
      rmSetAreaMaxBlobDistance(id, 40.0);
      rmSetAreaBaseHeight(id, 2.0);
      rmSetAreaHeightBlend(id, 2);
      rmSetAreaCoherence(id, 0.0);
      if(connect<0.7)
      rmAddConnectionArea(connectionID, id); 
      rmAddAreaConstraint(id, playerConstraint);
      rmSetAreaLocPlayer(id, i);
      if(skin==0)
      {
      rmSetAreaTerrainType(id, "GrassDirt50");
      rmAddAreaTerrainLayer(id, "GrassDirt25", 0, 12);
      }
      if(skin==1)
      {
      rmSetAreaTerrainType(id, "SandA");
      rmAddAreaTerrainLayer(id, "SandB", 0, 12);
      }
      if(skin==2)
      {
      rmSetAreaTerrainType(id, "SnowA");
      rmAddAreaTerrainLayer(id, "SnowB", 0, 12);
      }

      int secure=rmCreateArea("AreaSecure"+i);
      rmSetAreaSize(secure, .02, .02);
      rmSetAreaWarnFailure(secure, false);
      rmSetAreaBaseHeight(secure, 2.0);
      rmSetAreaHeightBlend(secure, 2);
      rmSetAreaLocPlayer(secure, i);
      if(skin==0)
      {
      rmSetAreaTerrainType(secure, "GrassDirt50");
      rmAddAreaTerrainLayer(secure, "GrassDirt25", 0, 3);
      }
      if(skin==1)
      {
      rmSetAreaTerrainType(secure, "SandA");
      rmAddAreaTerrainLayer(secure, "SandB", 0, 3);
      }
      if(skin==2)
      {
      rmSetAreaTerrainType(secure, "SnowA");
      rmAddAreaTerrainLayer(secure, "SnowB", 0, 3);
      }

      int pass=rmCreateArea("Passage"+i, centerID);
      rmSetAreaSize(pass, 0.02, 0.02);
      rmSetAreaLocPlayer(pass, i);
      rmSetAreaBaseHeight(pass, 2.0);
      rmSetAreaHeightBlend(pass, 2);
      rmAddAreaInfluenceSegment(pass, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i), 0.5, 0.5);
      if(skin==0)
      rmSetAreaTerrainType(pass, "GrassDirt50");
      if(skin==1)
      rmSetAreaTerrainType(pass, "SandA");
      if(skin==2)
      rmSetAreaTerrainType(pass, "SnowB");
   }

   // Text
   rmSetStatusText("",0.50);

      int join=rmCreateArea("Join", centerID);
      rmSetAreaSize(join, 0.05, 0.05);
      rmSetAreaBaseHeight(join, 2.0);
      rmSetAreaHeightBlend(join, 2);
      rmSetAreaLocation(join, 0.5, 0.5);
      if(skin==0)
      rmSetAreaTerrainType(join, "GrassDirt50");
      if(skin==1)
      rmSetAreaTerrainType(join, "SandA");
      if(skin==2)
      rmSetAreaTerrainType(join, "SnowB");

   // Build the areas.
   rmBuildAllAreas();
   if(connect<0.7)
   rmBuildConnection(connectionID);

   for(i=1; <cNumberPlayers*20)
   {
      // Beautification sub area.
      int id2=rmCreateArea("Patch"+i, centerID);
      rmSetAreaSize(id2, rmAreaTilesToFraction(50), rmAreaTilesToFraction(100));
      if(skin==0)
      rmSetAreaTerrainType(id2, "GrassDirt25");
      if(skin==1)
      rmSetAreaTerrainType(id2, "GrassDirt75");
      if(skin==2)
      rmSetAreaTerrainType(id2, "SnowGrass25");
      rmSetAreaWarnFailure(id2, false);
      rmSetAreaMinBlobs(id2, 1);
      rmSetAreaMaxBlobs(id2, 2);
      rmSetAreaMinBlobDistance(id2, 16.0);
      rmSetAreaMaxBlobDistance(id2, 40.0);
      rmSetAreaCoherence(id2, 0.0);
      rmAddAreaConstraint(id2, avoidImpassableLand);
      rmBuildArea(id2);
   }

   // Elev.
   int numTries=6*cNumberNonGaiaPlayers;
   int failCount=0;
   for(i=0; <numTries)
   {
      int elevID=rmCreateArea("elev"+i, centerID);
      rmSetAreaSize(elevID, rmAreaTilesToFraction(15), rmAreaTilesToFraction(80));
      rmSetAreaWarnFailure(elevID, false);
      rmAddAreaConstraint(elevID, playerConstraint);
      rmAddAreaToClass(elevID, rmClassID("classHill"));
      rmAddAreaConstraint(elevID, shortHillConstraint);
      rmAddAreaConstraint(elevID, avoidImpassableLand);
      if(rmRandFloat(0.0, 1.0)<0.5)
      if(skin==0)
      rmSetAreaTerrainType(elevID, "GrassB");
      if(skin==1)
      rmSetAreaTerrainType(elevID, "SandC");
      if(skin==2)
      rmSetAreaTerrainType(elevID, "SnowB");
      rmSetAreaBaseHeight(elevID, rmRandFloat(4.0, 7.0));
      rmSetAreaHeightBlend(elevID, 2);
      rmSetAreaMinBlobs(elevID, 1);
      rmSetAreaMaxBlobs(elevID, 5);
      rmSetAreaMinBlobDistance(elevID, 16.0);
      rmSetAreaMaxBlobDistance(elevID, 40.0);
      rmSetAreaCoherence(elevID, 0.0);

      if(rmBuildArea(elevID)==false)
      {
         // Stop trying once we fail 6 times in a row.
         failCount++;
         if(failCount==6)
            break;
      }
      else
         failCount=0;
   }

   // Slight Elevation
   numTries=10*cNumberNonGaiaPlayers;
   failCount=0;
   for(i=0; <numTries)
   {
      elevID=rmCreateArea("wrinkle"+i, centerID);
      rmSetAreaSize(elevID, rmAreaTilesToFraction(15), rmAreaTilesToFraction(120));
      rmSetAreaWarnFailure(elevID, false);
      rmAddAreaConstraint(elevID, shortAvoidImpassableLand);
      rmAddAreaConstraint(elevID, shortAvoidSettlement);
      rmAddAreaConstraint(elevID, shortHillConstraint);
      rmSetAreaBaseHeight(elevID, rmRandFloat(3.0, 5.0));
      rmSetAreaHeightBlend(elevID, 1);
      rmSetAreaMinBlobs(elevID, 1);
      rmSetAreaMaxBlobs(elevID, 3);
      rmSetAreaMinBlobDistance(elevID, 16.0);
      rmSetAreaMaxBlobDistance(elevID, 20.0);
      rmSetAreaCoherence(elevID, 0.0);

      if(rmBuildArea(elevID)==false)
      {
         // Stop trying once we fail 3 times in a row.
         failCount++;
         if(failCount==3)
            break;
      }
      else
         failCount=0;
   }
   // Text
   rmSetStatusText("",0.60);

   // Place starting settlements.
   // Close things....
   // TC
   rmPlaceObjectDefPerPlayer(startingSettlementID, true);

   // Towers.
   rmPlaceObjectDefPerPlayer(startingTowerID, true, 4);

   // Medium Objects
   int waterlifeType=rmRandInt(0,2);
   int sharkID=rmCreateObjectDef("shark");
   if(waterlifeType==0)
      rmAddObjectDefItem(sharkID, "shark", 1, 0.0);
   if(waterlifeType==1)
      rmAddObjectDefItem(sharkID, "whale", 1, 0.0);
   if(waterlifeType==2)
      rmAddObjectDefItem(sharkID, "hawksbill", 1, 0.0);
   rmSetObjectDefMinDistance(sharkID, 0.0);
   rmSetObjectDefMaxDistance(sharkID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(sharkID, sharkLand);
   rmPlaceObjectDefAtLoc(sharkID, 0, 0.5, 0.5, cNumberNonGaiaPlayers*0.5); 

   // Settlements.
   id=rmAddFairLoc("Settlement", false, true,  60, 80, 40, 40); /*back inside */
   rmAddFairLocConstraint(id, avoidImpassableLand);

   if(rmRandFloat(0,1)<0.75)
   {      
      id=rmAddFairLoc("Settlement", true, true, 90, 120, 70, 30);
      rmAddFairLocConstraint(id, avoidImpassableLand);
   }
   else
   {  
      id=rmAddFairLoc("Settlement", false, true,  60, 100, 40, 30);
      rmAddFairLocConstraint(id, avoidImpassableLand);
   }

   if(rmPlaceFairLocs())
   {
      id=rmCreateObjectDef("far settlement2");
      rmAddObjectDefItem(id, "Settlement", 1, 0.0);
      for(i=1; <cNumberPlayers)
      {
         for(j=0; <rmGetNumberFairLocs(i))
            rmPlaceObjectDefAtLoc(id, i, rmFairLocXFraction(i, j), rmFairLocZFraction(i, j), 1);
      }
   }


   // Straggler trees.
   rmPlaceObjectDefPerPlayer(stragglerTreeID, false, rmRandInt(2, 6));

   // Gold
   rmPlaceObjectDefPerPlayer(startingGoldID, false);

   // Text
   rmSetStatusText("",0.70);

   // cows
   rmPlaceObjectDefPerPlayer(closeCowsID, true);

   // Berries  
   rmPlaceObjectDefPerPlayer(closeBerriesID, true);

   // Boar.
   bonusChance=rmRandFloat(0, 1);
   if(bonusChance<0.7)
   rmPlaceObjectDefPerPlayer(closeBoarID, false);
   
   // Medium things....
   // Gold
   rmPlaceObjectDefPerPlayer(mediumGoldID, false, rmRandInt(1, 2));

   // player fish
   rmPlaceObjectDefPerPlayer(playerFishID, false, 3);

   rmPlaceObjectDefPerPlayer(mediumWalrusID, false);

   rmPlaceObjectDefPerPlayer(mediumDeerID, false, 3);
            
   // Far things.

   // Gold.
   rmPlaceObjectDefPerPlayer(farGoldID, false, rmRandInt(2, 3));

   // Relics.
   rmPlaceObjectDefPerPlayer(relicID, false);

   // Hawks
   rmPlaceObjectDefPerPlayer(farhawkID, false, 2);

   if(skin==2)
   {   
      if(border>0.8)
      {
      // Bonus huntable.
      rmPlaceObjectDefPerPlayer(bonusHuntableID, false, 5);

      // Herds.
      rmPlaceObjectDefPerPlayer(farCowID, false, rmRandInt(5,8));
      }
      else
      {
      // Bonus huntable.
      rmPlaceObjectDefPerPlayer(bonusHuntableID, false, 2);

      // Herds.
      rmPlaceObjectDefPerPlayer(farCowID, false, rmRandInt(3,4));
      }
   }
   else
   {
   // Bonus huntable.
   rmPlaceObjectDefPerPlayer(bonusHuntableID, false, 2);

   // Herds.
   rmPlaceObjectDefPerPlayer(farCowID, false, rmRandInt(3,4));
   }

   // Predators
   rmPlaceObjectDefPerPlayer(farPredatorID, false, 2);
   
   // Text
   rmSetStatusText("",0.80);

   // Second bonus huntable stuff.
   if(rmRandFloat(0,1)<0.75)
      rmPlaceObjectDefPerPlayer(bonusWalrus, false, 1);
   else if(cNumberNonGaiaPlayers < 8)
      rmPlaceObjectDefPerPlayer(bonusWalrus, false, 2);
   else
      rmPlaceObjectDefPerPlayer(bonusWalrus, false, 1);

   // Forest.
   int classForest=rmDefineClass("forest");
   int forestObjConstraint=rmCreateTypeDistanceConstraint("forest obj", "all", 6.0);
   int forestConstraint=rmCreateClassDistanceConstraint("forest v forest", rmClassID("forest"), 20.0);
   int forestSettleConstraint=rmCreateClassDistanceConstraint("forest settle", rmClassID("starting settlement"), 20.0);
   int count=0;
   numTries=8*cNumberNonGaiaPlayers;
   failCount=0;
   for(i=0; <numTries)
   {
      int forestID=rmCreateArea("forest"+i, centerID);
      rmSetAreaSize(forestID, rmAreaTilesToFraction(50), rmAreaTilesToFraction(140));
      rmSetAreaWarnFailure(forestID, false);
      if(skin==0)
      rmSetAreaForestType(forestID, "mixed oak forest");
      if(skin==1)
      rmSetAreaForestType(forestID, "palm forest");
      if(skin==2)
      rmSetAreaForestType(forestID, "snow pine forest");
      rmAddAreaConstraint(forestID, forestSettleConstraint);
      rmAddAreaConstraint(forestID, forestObjConstraint);
      rmAddAreaConstraint(forestID, forestConstraint);
      rmAddAreaConstraint(forestID, avoidImpassableLand);
      rmAddAreaToClass(forestID, classForest);
      
      rmSetAreaMinBlobs(forestID, 1);
      rmSetAreaMaxBlobs(forestID, 4);
      rmSetAreaMinBlobDistance(forestID, 16.0);
      rmSetAreaMaxBlobDistance(forestID, 25.0);
      rmSetAreaCoherence(forestID, 0.0);

      // Hill trees?
      if(rmRandFloat(0.0, 1.0)<0.2)
         rmSetAreaBaseHeight(forestID, rmRandFloat(3.0, 4.0));

      if(rmBuildArea(forestID)==false)
      {
         // Stop trying once we fail 3 times in a row.
         failCount++;
         if(failCount==3)
            break;
      }
      else
         failCount=0;
   }

   int fishID=rmCreateObjectDef("fish");
   fishType=rmRandInt(0,2);
   if(fishType==0)
   rmAddObjectDefItem(fishID, "fish - salmon", 3, 10.0);
   if(fishType==1)
   rmAddObjectDefItem(fishID, "fish - perch", 3, 10.0);
   if(fishType==2)
   rmAddObjectDefItem(fishID, "fish - mahi", 3, 10.0);
   rmSetObjectDefMinDistance(fishID, 0.0);
   rmSetObjectDefMaxDistance(fishID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(fishID, fishVsFishID);
   rmAddObjectDefConstraint(fishID, fishLand);
   rmPlaceObjectDefAtLoc(fishID, 0, 0.5, 0.5, 10*cNumberNonGaiaPlayers);

   int orcaLand = rmCreateTerrainDistanceConstraint("orca land", "land", true, 20.0);
   int orcaVsOrca=rmCreateTypeDistanceConstraint("orca v orca", "orca", 20.0);
   int orcaID=rmCreateObjectDef("orca");
   rmAddObjectDefItem(orcaID, "orca", 1, 0.0);
   rmSetObjectDefMinDistance(orcaID, 0.0);
   rmSetObjectDefMaxDistance(orcaID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(orcaID, orcaLand);
   rmAddObjectDefConstraint(orcaID, orcaVsOrca);
   rmAddObjectDefConstraint(orcaID, edgeConstraint);
   rmPlaceObjectDefAtLoc(orcaID, 0, 0.5, 0.5, cNumberNonGaiaPlayers*0.5);

   // Text
   rmSetStatusText("",0.90);

   fishID=rmCreateObjectDef("fish2");
   rmAddObjectDefItem(fishID, "fish - perch", 1, 0.0);
   rmSetObjectDefMinDistance(fishID, 0.0);
   rmSetObjectDefMaxDistance(fishID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(fishID, fishVsFishID);
/*   rmAddObjectDefConstraint(fishID, fishEdge); */
   rmAddObjectDefConstraint(fishID, fishLand);
   rmPlaceObjectDefAtLoc(fishID, 0, 0.5, 0.5, 4*cNumberNonGaiaPlayers);

   // Random trees.
   rmPlaceObjectDefAtLoc(randomTreeID, 0, 0.5, 0.5, 20*cNumberNonGaiaPlayers);
   
   // Rocks
   int avoidAll=rmCreateTypeDistanceConstraint("avoid all", "all", 6.0);
   int rockID=rmCreateObjectDef("rock");
   if(skin==0)
   rmAddObjectDefItem(rockID, "rock limestone sprite", 1, 0.0);
   if(skin==1)
   rmAddObjectDefItem(rockID, "rock sandstone sprite", 1, 0.0);
   if(skin==2)
   rmAddObjectDefItem(rockID, "rock granite sprite", 1, 0.0);
   rmSetObjectDefMinDistance(rockID, 0.0);
   rmSetObjectDefMaxDistance(rockID, rmXFractionToMeters(0.5));
   //rmAddObjectDefConstraint(rockID, avoidRock);
   rmAddObjectDefConstraint(rockID, avoidAll);
   rmAddObjectDefConstraint(rockID, avoidImpassableLand);
   rmPlaceObjectDefAtLoc(rockID, 0, 0.5, 0.5, 40*cNumberNonGaiaPlayers);

      int nearshore=rmCreateTerrainMaxDistanceConstraint("seaweed near shore", "land", true, 12.0);
      int farshore = rmCreateTerrainDistanceConstraint("seaweed far from shore", "land", true, 8.0);
      int kelpID=rmCreateObjectDef("seaweed");
      rmAddObjectDefItem(kelpID, "seaweed", 5, 3.0);
      rmSetObjectDefMinDistance(kelpID, 0.0);
      rmSetObjectDefMaxDistance(kelpID, rmXFractionToMeters(0.5));
      rmAddObjectDefConstraint(kelpID, avoidAll);
      rmAddObjectDefConstraint(kelpID, nearshore);
      rmAddObjectDefConstraint(kelpID, farshore);
      rmPlaceObjectDefAtLoc(kelpID, 0, 0.5, 0.5, 8*cNumberNonGaiaPlayers);

      int kelp2ID=rmCreateObjectDef("seaweed 2");
      rmAddObjectDefItem(kelp2ID, "seaweed", 2, 3.0);
      rmSetObjectDefMinDistance(kelp2ID, 0.0);
      rmSetObjectDefMaxDistance(kelp2ID, rmXFractionToMeters(0.5));
      rmAddObjectDefConstraint(kelp2ID, avoidAll);
      rmAddObjectDefConstraint(kelp2ID, nearshore);
      rmAddObjectDefConstraint(kelp2ID, farshore);
      rmPlaceObjectDefAtLoc(kelp2ID, 0, 0.5, 0.5, 12*cNumberNonGaiaPlayers);


   // Text
   rmSetStatusText("",1.00); 
}